export const colorLegend = (selection, props) => {
  const {                      
    colorScale,                
    circleRadius,
    spacing,                   
    textOffset,
    backgroundRectWidth,
    onClick,
    selectedColorValue
  } = props;      
  
  let n = colorScale.domain().length; 
  if(n > 15 && (typeof colorScale.domain()[0] !== "number")) {
    selection.selectAll('rect').remove();
    return;
  }
  
  let data = colorScale.domain();
  if(n > 15) {
    let colors = Array();
    const factor = Math.floor(n / 12);

    for(var i = 0; i < n; i = i+factor) {
      colors.push(data[i])
    }
    if(i !== (n-1))
      colors.push(data[n-1]);
    
    data = colors;
  }
  
  n = (n > 12) ? 15 : n;
  
  // Background Rectangle
  const backgroundRect = selection.selectAll('rect')
    .data([null]);             
  backgroundRect.enter().append('rect')
    .merge(backgroundRect)
      .attr('x', -circleRadius * 2)   
      .attr('y', -circleRadius * 2)   
      .attr('rx', circleRadius * 2)   
      .attr('width', backgroundRectWidth)
      .attr('height', spacing * n + circleRadius * 2) 
      .attr('fill', 'white')
      .attr('opacity', 0.8);
  

  // Groups For Legend
  const groups = selection.selectAll('.legendFont')
    .data(data);
  
  const groupsEnter = groups
    .enter().append('g')
      .attr('class', 'legendFont');
  
  groupsEnter
    .merge(groups)
      .attr('transform', (d, i) =>    
        `translate(0, ${i * spacing})`  
      )
      .attr('opacity', d =>
        (!selectedColorValue || d === selectedColorValue)
          ? 1
          : 0.2
      )
  	  .on('click', d => onClick(d === selectedColorValue ? null : d));

  groups.exit().remove();
  
  groupsEnter.append('circle')
    .merge(groups.select('circle')) 
      .attr('r', circleRadius)
      .attr('fill', colorScale)
 
  groupsEnter.append('text')
    .merge(groups.select('text'))   
      .text(d => d)
      .attr('dy', '0.32em')
      .attr('x', textOffset);
}
