import {
  select,
  csv,
  interpolateViridis,
  schemeSpectral,
  scaleOrdinal,
  scaleLinear,
  extent,
  axisLeft,
  axisBottom
} from 'd3';
import { dropdownMenu } from './dropdownMenu';
import { scatterPlot } from './scatterPlot';
import { colorLegend } from './colorLegend';
const svg = select('svg');
const colorLegendG = svg.append('g');

const width = +svg.attr('width');
const height = +svg.attr('height');

let data;
let xColumn;
let yColumn;
let category;
let selectedColorValue;

const onXColumnClicked = column => {
  xColumn = column; 
  render();
};

const onYColumnClicked = column => {
  yColumn = column;
  render();
};

const onCategoryClicked = selectedCategory => {
  category = selectedCategory;
  render();
}

const onColorClicked = color => {
  selectedColorValue = color;
  render();
}

const render = () => {
  
  const colorValue = d => d[category];
  const colorScale = scaleOrdinal();
  colorScale
    .domain(data.map(colorValue))
    .domain(colorScale.domain().sort().reverse());
  
  const n = colorScale.domain().length;
  if(n > 12) {
    colorScale.range(colorScale.domain().map((d, i) =>
      interpolateViridis(i / (n + 1))
    ));
  } else {
    colorScale.range(schemeSpectral[colorScale.domain().length]);
  }
  
  select('#x-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onXColumnClicked,
      selectedOption: xColumn
    });
  
  select('#y-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onYColumnClicked,
      selectedOption: yColumn
    });
  
  select('#category-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onCategoryClicked,
      selectedOption: category,
  });
  
  colorLegendG.attr('transform', 'translate(824, 30)');
  colorLegendG.call(colorLegend,{
    colorScale,
    circleRadius: 10,
    spacing: 21,
    textOffset: 16,
    backgroundRectWidth:150,
    onClick: onColorClicked,
    selectedColorValue,
  });
  
  svg.call(scatterPlot, {
    xValue: d => d[xColumn],
    xAxisLabel: xColumn,
    yValue: d => d[yColumn],
    circleRadius: 10,
    yAxisLabel: yColumn,
    margin: { top: 10, right: 160, bottom: 88, left: 130 },
    selectedColorValue,
    colorScale,
    colorValue,
    category,
    width,
    height,
    data
  });
};

csv('https://vizhub.com/curran/datasets/auto-mpg.csv')
  .then(loadedData => {
    data = loadedData;
    data.forEach(d => {
      d.mpg = +d.mpg;
      d.cylinders = +d.cylinders;
      d.displacement = +d.displacement;
      d.horsepower = +d.horsepower;
      d.weight = +d.weight;
      d.acceleration = +d.acceleration;
      d.year = +d.year;  
    });
  
    category = data.columns[8];
    xColumn = data.columns[4];
    yColumn = data.columns[0];
    render();
  });